////////////
//INCLUDES//
////////////
#include <iostream.h>
#include "ami_udp.h"

SOCKET OpenSocket(void)
{
	SOCKET Socket;

	/////////////////////
	//Create the socket//
	/////////////////////
	Socket = socket(AF_INET, SOCK_DGRAM, 0);
 
	if (Socket == INVALID_SOCKET)
	{
		cout << "Socket Failed: " << WSAGetLastError() << endl;
		closesocket(Socket); 
		return - 1; 
	}

	return Socket;
}

BOOL SetStandardSocketOpts(SOCKET Socket)
{
	int status;
	BOOL bOptVal = TRUE;

	//////////////////////
	//Set Socket Options//
	//////////////////////

	status = setsockopt(Socket,					//A valid socket
						SOL_SOCKET,				//
						SO_BROADCAST,			//So socket can be used to broadcast over 255.255.255.255
						(char far*)&bOptVal,	//bOptVal = true
						sizeof(BOOL));

	if (status == SOCKET_ERROR) 
	{ 
		cout << "SetSockOpt SO_BROADCAST failed: " << WSAGetLastError() << endl; 
		return -1; 
	}

	return 1;
}

BOOL BindSocketToPort(SOCKET Socket, SOCKADDR_IN *LocalSin, unsigned short LocalUDPPort)
{
	int LocalSinLen;

	LocalSinLen = sizeof(SOCKADDR_IN);

	///////////////////
	//Name the socket//
	///////////////////
	LocalSin->sin_family = AF_INET;
	LocalSin->sin_addr.s_addr = INADDR_ANY;
	LocalSin->sin_port = (unsigned short)htons(LocalUDPPort); 

	while(1)
	{
		if (bind(Socket, (SOCKADDR *) LocalSin, sizeof(SOCKADDR_IN)) == SOCKET_ERROR)
		{
			cout << "bind failed: " << WSAGetLastError() << endl;
			return FALSE;
		}
		else
		{
			//Retrieve the assigned port number for later use
			getsockname(Socket, (SOCKADDR *) LocalSin, &LocalSinLen);
		}
		if (ntohs(LocalSin->sin_port) != 2639) //Disallow use of the reserved AMINet Port
			break;
	}

	//return the IP Address and Port number assigned to the socket for later use
	return TRUE;
}

BOOL ReceiveUDPMessage(SOCKET Socket, char * RecvBuffer)
{

	int status;

	/////////////////////
	//Receive a message//
	/////////////////////
	status = recvfrom(Socket,					//bound socket
					  RecvBuffer,				//buffer for incoming data
					  MAXMSGLEN,				//length of buf
					  (int)NULL,				//option flags
					  0,						//optional buffer for source addr
					  0);						//optional length of this buffer

 	if (status == SOCKET_ERROR)
	{
		status = WSAGetLastError();
		if (status != 10004) 
		{
			cout << "Recv Error Winsock Error: " << status << endl;
			return FALSE;
		}
	}

	return TRUE;
}

BOOL SendUDPMessage(SOCKET Socket, char *SendBuffer, SOCKADDR_IN RemoteSin)
{
	int status;

	//////////////////
	//Send a message//
	//////////////////
	status = sendto(Socket, 
					(char *)SendBuffer,			//buffer containing outgoing data
					(int)strlen(SendBuffer),	//length of data to send
					(int)NULL,					//option flags
					(SOCKADDR *)&RemoteSin,		//ptr to target addr
					sizeof(SOCKADDR_IN));		//length of addr structure

	if (status == SOCKET_ERROR)
	{
		cout << "sendto failed: " << WSAGetLastError() << endl;
		return FALSE;
	}

	return TRUE;
}

void AddAMINetChecksum (char *buffer)
{
	char sChecksumString[5];
	int nChecksum = 0;
	int i;

	for (i = 1; i < (int)strlen(buffer); i++)
	{
		nChecksum += buffer[i];
	}

	if (nChecksum >= 65536)
	{
		sChecksumString[0] = (char)0xFE;
		sChecksumString[1] = (unsigned char)((nChecksum & 0xFF0000) >> 16);
		sChecksumString[2] = (unsigned char)((nChecksum & 0xFF00) >> 8);
		sChecksumString[3] = (unsigned char)(nChecksum & 0xFF);
		sChecksumString[4] = '\0';
	}
	else if (nChecksum >= 250)
	{
		sChecksumString[0] = (char)0xFF;
		sChecksumString[1] = (unsigned char)((nChecksum & 0xFF00) >> 8);
		sChecksumString[2] = (unsigned char)(nChecksum & 0xFF);
		sChecksumString[3] = '\0';
	}
	else
	{
		sChecksumString[0] = (unsigned char)nChecksum;
		sChecksumString[1] = '\0';
	}
	strcat(buffer, sChecksumString);
}
